@@ -4,8 +4,8 @@ pub mod macros {
     /// as_to!() Example
     /// convert type
     /// ```rust
-    /// let s = as_to!(5.,i64);
-    /// println!("{:?}",s);
+    ///  let s = as_to!(5., i64);
+    ///  assert_eq!(5, s);
     /// ```
     #[macro_export]
     macro_rules! as_to {
@@ -84,7 +84,7 @@ pub mod macros {
     /// a to the power of b
     /// ```rust
     /// let p = powf!(2.,2.);
-    /// println!("{:?}",p);
+    /// assert_eq!(p,4.0);
     /// ```
     #[macro_export]
     macro_rules! powf {
@@ -93,10 +93,11 @@ pub mod macros {
         };
     }
     /// arg!() Example
-    ///get argument and collect to Vec<String>
+    ///get argument and collect to Vec\<String\>
     /// ```rust
+    /// //cargo run -- -n 100
     /// let arg = args!();
-    /// println!("{:?}",arg);
+    /// assert_eq!(arg,vec![format!("-n"),format!("100")]);
     /// ```
     #[macro_export]
     macro_rules! args {
@@ -120,10 +121,10 @@ pub mod macros {
         }};
     }
     /// split_to_vec!() Example
-    /// a:&str split by b:&str and collect to Vec<String>
+    /// a:&str split by b:&str and collect to Vec\<String\>
     /// ```rust
     /// let s = split_to_vec!("aa.bb",".");
-    /// println!("{:?}",s);
+    /// assert_eq!(s,vec![format!("aa"),format!("bb")]);
     /// ```
     #[macro_export]
     macro_rules! split_to_vec {
@@ -136,10 +137,10 @@ pub mod macros {
         };
     }
     /// read_csv!() Example
-    /// read .csv file and return Vec<Vec<String>>
+    /// read .csv file and return Vec\<Vec\<String\>\>
     /// ```rust
-    /// let s = read_csv!("./data.csv");
-    /// println!("{:?}",s);
+    ///let s = read_csv!("./data.csv");
+    ///assert_eq!(s,vec![vec![format!("a"), format!("b"), format!("c")],vec![format!("1"), format!("2"), format!("3")],vec![format!("10"), format!("20"), format!("30")]]);
     /// ```
     #[macro_export]
     macro_rules! read_csv {
@@ -153,12 +154,81 @@ pub mod macros {
             data_each_vec
         }};
     }
+    /// sorted!() Example
+    /// return sorted new Vec,
+    /// type can be Vec\<i32\> Vec\<i64\> Vec\<i128\> Vec\<f32\> Vec\<f64\>   
+    /// ```rust
+    /// let s1 = sorted!(vec![1.2, 2.6, 0.2]);
+    /// let s2 = sorted!(vec![8, 1_i128, 5_i128]);
+    /// assert_eq!(s1,vec![0.2,1.2,2.6]);
+    /// assert_eq!(s2,vec![1,5,8]);
+    /// ```
+    #[macro_export]
+    macro_rules! sorted {
+        ($vec:expr) => {{
+            let mut vec = $vec.clone();
+            vec.sort_by(|a, b| a.partial_cmp(b).unwrap());
+            vec
+        }};
+    }
+    /// deduped_sorted!() Example
+    /// return sorted and deduped new Vec,
+    /// type can be Vec\<i32\> Vec\<i64\> Vec\<i128\> Vec\<f32\> Vec\<f64\>   
+    /// ```rust
+    /// let s1 = deduped_sorted!(vec![1.2, 1.2,2.6, 0.2]);
+    /// let s2 = deduped_sorted!(vec![8, 1_i128, ,8,5_i128]);
+    /// assert_eq!(s1,vec![0.2,1.2,2.6]);
+    /// assert_eq!(s2,vec![1,5,8]);
+    /// ```
+    #[macro_export]
+    macro_rules! deduped_sorted {
+        ($vec:expr) => {{
+            let mut vec1 = $vec.clone();
+            let mut vec2 = sorted!(vec1);
+            vec2.dedup();
+            vec2
+        }};
+    }
+    /// vec_parse!() Example
+    /// parse vec item to f64,
+    /// type can be Vec\<i32\> Vec\<i64\> Vec\<i128\> Vec\<f32\> Vec\<f64\>,
+    /// return Vec\<f64\>
+    /// ```rust
+    ///let v1 = vec_parse!(vec!["15.", "2.9"]);
+    ///let v2 = vec_parse!(vec!["15", "2"]);
+    ///let v3 = vec_parse!(vec![".15", ".2"]);
+    ///assert_eq!(vec![15.0, 2.9], v1);
+    ///assert_eq!(vec![15.0, 2.0], v2);
+    ///assert_eq!(vec![0.15, 0.2], v3);
+    /// ```
+    #[macro_export]
+    macro_rules! vec_parse {
+        ($vec:expr) => {{
+            let mut v2: Vec<f64> = Vec::new();
+
+            if $vec.len() > 0 {
+                match &$vec[0].parse::<f64>() {
+                    Ok(r) => {
+                        let vec1 = $vec.clone();
+                        v2 = vec1
+                            .iter()
+                            .map(|x| x.to_string().parse::<f64>().unwrap())
+                            .collect::<Vec<_>>();
+                    }
+                    Err(e) => {}
+                }
+            }
+            v2
+        }};
+    }
+
     pub fn run() {
-        // println!("{:?}", powf!(4., 2.));
-        // println!("{:?}", split_to_vec!("dnrops.gitee.io", "."));
-        // println!("{:?}", args!()); //get argument Vec<String>
-        // println!("{:?}", input!()); //get input from treminal to String
-        println!("{:?}", read_csv!("./data.csv"));
+        let v1 = vec_parse!(vec!["15.", "2.9"]);
+        let v2 = vec_parse!(vec!["15", "2"]);
+        let v3 = vec_parse!(vec![".15", ".2"]);
+        assert_eq!(vec![15.0, 2.9], v1);
+        assert_eq!(vec![15.0, 2.0], v2);
+        assert_eq!(vec![0.15, 0.2], v3);
     }
 }
 
